@@ -3,6 +3,7 @@
 #include "smallUsefulFunctions.h"
 #include "configmanagerinterface.h"
 #include "utilsSystem.h"
+#include "execprogram.h"
 
 #include "userquickdialog.h"
 
@@ -127,7 +128,7 @@ QString CommandInfo::getProgramName() const
 
 QString CommandInfo::getProgramNameUnquoted() const
 {
-    return getProgramNameUnquoted(commandLine);
+	return getProgramNameUnquoted(commandLine);
 }
 
 ExpandingOptions::ExpandingOptions(const QFileInfo &mainFile, const QFileInfo &currentFile, const int currentLine): mainFile(mainFile), currentFile(currentFile), currentLine(currentLine), nestingDeep(0), canceled(false)
@@ -270,10 +271,6 @@ BuildManager::BuildManager(): processWaitedFor(nullptr)
 {
 	initDefaultCommandNames();
 	connect(this, SIGNAL(commandLineRequested(QString, QString *, bool *)), SLOT(commandLineRequestedDefault(QString, QString *, bool *)));
-
-	m_stopBuildAction = new QAction(getRealIcon("stop"), tr("Stop Compile"), this);
-	connect(m_stopBuildAction, SIGNAL(triggered()), this, SLOT(killCurrentProcess()));
-	m_stopBuildAction->setEnabled(false);
 }
 
 BuildManager::~BuildManager()
@@ -422,8 +419,8 @@ void BuildManager::checkOSXElCapitanDeprecatedPaths(QSettings &settings, const Q
 		}
 	}
 #else
-    Q_UNUSED(settings)
-    Q_UNUSED(commands)
+	Q_UNUSED(settings)
+	Q_UNUSED(commands)
 #endif
 }
 
@@ -513,9 +510,7 @@ QStringList BuildManager::parseExtendedCommandLine(QString str, const QFileInfo
 					(*createCommand) += str.at(i);
 					i++;
 				}
-				bool useCurrentFile = command.startsWith("c:");
-				const QFileInfo &selectedFile = (useCurrentFile && !currentFile.fileName().isEmpty()) ? currentFile : mainFile;
-				if (useCurrentFile) command = command.mid(2);
+				QFileInfo selectedFile = parseExtendedSelectFile(command, mainFile, currentFile);
 				bool absPath = command.startsWith('a');
 				//check only sane commands
 				if (command == "ame")
@@ -583,12 +578,54 @@ QStringList BuildManager::parseExtendedCommandLine(QString str, const QFileInfo
 	return result;
 }
 
+/*
+ * Select a file which provides the pathname parts used by the "ame" expansions. Currently we can select
+ * one of the following files:
+ *
+ * - Master (root) .tex file (default).
+ * - Current .tex file. Selected by the c: prefix.
+ * - A file with the same complete basename as the master file and a chosen extension. The search for this
+ *   file is done in the master file directory and then the extra PDF directories. Selected by the p{ext}:
+ *   prefix.
+ *
+ * TODO: If selector ?p{ext}: is not flexible enough then maybe we should implement another selector:
+ * ?f{regexp_with_basename}:
+ *
+ * It will be processed like this:
+ *
+ * 1. regexp_with_basename undergoes %-token replacement
+ *    %m is replaced by the complete basename of the master file.
+ *    %% is replaced by %
+ * 2. The expression from 1 is used to search the master file directory, the current file
+ *    directory and the extra PDF directories.
+ * 3. If step 2 finds a matching file it is used as a selected file. If step 2 does not
+ *    find a file, then some reasonable default is used.
+ */
+QFileInfo BuildManager::parseExtendedSelectFile(QString &command, const QFileInfo &mainFile, const QFileInfo &currentFile)
+{
+	QFileInfo selectedFile;
+	QRegExp rxPdf("^p\\{([^{}]+)\\}:");
+
+	if (command.startsWith("c:")) {
+		selectedFile = currentFile.fileName().isEmpty() ? mainFile : currentFile;
+		command = command.mid(2);
+	} else if (rxPdf.indexIn(command) != -1) {
+		QString compiledFilename = mainFile.completeBaseName() + '.' + rxPdf.cap(1);
+		QString compiledFound = findCompiledFile(compiledFilename, mainFile);
+		selectedFile = QFileInfo(compiledFound);
+		command = command.mid(rxPdf.matchedLength());
+	} else {
+		selectedFile = mainFile;
+	}
+	return selectedFile;
+}
+
 /*!
  * \brief extracts the
  * \param s
  * \param stdOut output parameter
  * \param stdErr output parameter
- * \return a copy of s truncated to the first occurence of an output redirection
+ * \return a copy of s truncated to the first occurrence of an output redirection
  */
 QString BuildManager::extractOutputRedirection(const QString &commandLine, QString &stdOut, QString &stdErr)
 {
@@ -602,7 +639,7 @@ QString BuildManager::extractOutputRedirection(const QString &commandLine, QStri
 
 QString addPathDelimeter(const QString &a)
 {
-    return ((a.endsWith("/") || a.endsWith("\\")) ? a : (a + QDir::separator()));
+	return ((a.endsWith("/") || a.endsWith("\\")) ? a : (a + QDir::separator()));
 }
 
 QString BuildManager::findFileInPath(QString fileName)
@@ -633,7 +670,7 @@ QString W32_FileAssociation(QString ext)
 			result = QString::fromLatin1(buf);
 			result.replace("%1", "?am" + ext);
 			//QMessageBox::information(0,result,result,0);
-		};
+		}
 	}
 	FreeLibrary(mod);
 	return result;
@@ -714,7 +751,7 @@ QString getMiKTeXBinPathInternal()
 	}
 
 	// search all program file paths
-    if (mikPath.isEmpty()) {
+	if (mikPath.isEmpty()) {
 		mikPath = QDir::toNativeSeparators(findSubDir(getProgramFilesPaths(), "*miktex*", "miktex\\bin\\"));
 	}
 
@@ -932,10 +969,10 @@ ExpandedCommands BuildManager::expandCommandLine(const QString &str, ExpandingOp
 				UtilsUi::txsInformation(tr("You have used txs:///command[... or txs:///command{... modifiers, but we only support modifiers of the form txs:///command/[... or txs:///command/{... with an slash suffix to keep the syntax purer."));
 				modifiers.clear();
 			}
-            if (options.override.removeAll) {
-                parameters.clear();
-                modifiers.clear();
-            }
+			if (options.override.removeAll) {
+				parameters.clear();
+				modifiers.clear();
+			}
 
 			bool user;
 			QString cmd = getCommandLine(cmdName, &user);
@@ -952,8 +989,8 @@ ExpandedCommands BuildManager::expandCommandLine(const QString &str, ExpandingOp
 			int space = cmd.indexOf(' ');
 			if (space == -1) space = cmd.size();
 			if (cmd.startsWith(TXS_CMD_PREFIX) && internalCommands.contains(cmd.left(space))) {
-                QStringList exp=parseExtendedCommandLine(cmd, options.mainFile, options.currentFile, options.currentLine);
-                res.commands << CommandToRun(exp.first()+" "+parameters);
+				QStringList exp=parseExtendedCommandLine(cmd, options.mainFile, options.currentFile, options.currentLine);
+				res.commands << CommandToRun(exp.first()+" "+parameters);
 				res.commands.last().parentCommand = res.commands.last().command;
 				if (user) res.commands.last().flags |= RCF_CHANGE_PDF;
 				continue;
@@ -1061,7 +1098,7 @@ RunCommandFlags BuildManager::getSingleCommandFlags(const QString &subcmd) const
 	int result = 0;
 	if (similarCommandInList(subcmd, latexCommands)) result |= RCF_COMPILES_TEX;
 	if (similarCommandInList(subcmd, pdfCommands)) result |= RCF_CHANGE_PDF;
-	if (rerunnableCommands.contains(subcmd)) result |= RCF_RERUNNABLE;
+	if (similarCommandInList(subcmd, rerunnableCommands)) result |= RCF_RERUNNABLE;
 	if (stdoutCommands.contains(subcmd)) result |= RCF_SHOW_STDOUT;
 	bool isAcrobat = false;
 #ifdef Q_OS_WIN
@@ -1069,7 +1106,7 @@ RunCommandFlags BuildManager::getSingleCommandFlags(const QString &subcmd) const
 #endif
 
 	if (viewerCommands.contains(subcmd) && !isAcrobat && singleViewerInstance) result |= RCF_SINGLE_INSTANCE;
-    return static_cast<RunCommandFlags>(result);
+	return static_cast<RunCommandFlags>(result);
 }
 
 bool BuildManager::hasCommandLine(const QString &program)
@@ -1379,7 +1416,7 @@ void BuildManager::readSettings(QSettings &settings)
 		if (!deprecatedUserToolNames[i].endsWith("!!!CONVERTED!!!")) {
 			QString cmd = deprecatedUserToolCommands[i];
 			cmd.replace(DEPRECACTED_TMX_INTERNAL_PDF_VIEWER, CMD_VIEW_PDF_INTERNAL);
-            CommandInfo &ci = registerCommand(QString("user%1").arg(i), "", deprecatedUserToolNames[i], "", "", nullptr, true);
+			CommandInfo &ci = registerCommand(QString("user%1").arg(i), "", deprecatedUserToolNames[i], "", "", nullptr, true);
 			ci.commandLine = cmd;
 			userToolOrder << ci.id;
 			userToolDisplayNames << ci.displayName;
@@ -1390,7 +1427,7 @@ void BuildManager::readSettings(QSettings &settings)
 		QString temp = settings.value(QString("User/Tool%1").arg(i), "").toString();
 		if (!temp.isEmpty()) {
 			temp.replace(DEPRECACTED_TMX_INTERNAL_PDF_VIEWER, CMD_VIEW_PDF_INTERNAL);
-            CommandInfo &ci = registerCommand(QString("userold%1").arg(i), "", settings.value(QString("User/ToolName%1").arg(i)).toString(), "", "", nullptr, true);
+			CommandInfo &ci = registerCommand(QString("userold%1").arg(i), "", settings.value(QString("User/ToolName%1").arg(i)).toString(), "", "", nullptr, true);
 			ci.commandLine  = temp;
 			userToolOrder << ci.id;
 			userToolDisplayNames << ci.displayName;
@@ -1398,7 +1435,7 @@ void BuildManager::readSettings(QSettings &settings)
 			settings.remove(QString("User/ToolName%1").arg(i));
 		}
 	}
-    int md = dvi2pngMode;
+	int md = dvi2pngMode;
 #ifdef NO_POPPLER_PREVIEW
 	if (md == DPM_EMBEDDED_PDF)
 		md = -1;
@@ -1601,20 +1638,19 @@ void BuildManager::emitEndRunningSubCommandFromProcessX(int)
 ProcessX *BuildManager::firstProcessOfDirectExpansion(const QString &command, const QFileInfo &mainFile, const QFileInfo &currentFile, int currentLine,bool nonstop)
 {
 	ExpandingOptions options(mainFile, currentFile, currentLine);
-    if(nonstop){
-        options.nestingDeep=1; // tweak to avoid pop-up error messages
-    }
+	if(nonstop){
+		options.nestingDeep=1; // tweak to avoid pop-up error messages
+	}
 	ExpandedCommands expansion = expandCommandLine(command, options);
-    if (options.canceled) return nullptr;
+	if (options.canceled) return nullptr;
 
-    if (expansion.commands.isEmpty()) { return nullptr; }
+	if (expansion.commands.isEmpty()) { return nullptr; }
 
-    foreach(CommandToRun elem,expansion.commands){
-        if(elem.command.isEmpty()){
-            return nullptr; // error in command expansion
-        }
-
-    }
+	foreach(CommandToRun elem,expansion.commands) {
+		if(elem.command.isEmpty()) {
+			return nullptr; // error in command expansion
+		}
+	}
 
 	return newProcessInternal(expansion.commands.first().command, mainFile);
 }
@@ -1642,26 +1678,24 @@ ProcessX *BuildManager::newProcessInternal(const QString &cmd, const QFileInfo &
 
 bool BuildManager::waitForProcess(ProcessX *p)
 {
+	REQUIRE_RET(p, false);
 	REQUIRE_RET(!processWaitedFor, false);
-	processWaitedFor = p;
-	m_stopBuildAction->setEnabled(true);
-	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
-	QTime time;
-	time.start();
-	while (p && p->isRunning()) {
-		if (time.elapsed() > 2000)
-			qApp->instance()->processEvents(QEventLoop::AllEvents);
-		else
-			qApp->instance()->processEvents(QEventLoop::ExcludeUserInputEvents);
-		// workaround for high CPU consumption when waiting
-		// TODO: replace this loop by notification mechansim
-		ThreadBreaker::msleep(1);
+	// Waiting on a Qt event loop avoids spinlock and high CPU usage, and allows user interaction
+	// and UI responsiveness while compiling.
+	// We have to check the process running state before we start waiting for processFinished
+	// because it is possible that the process has already ended and we would wait forever.
+	// We have to start listening for processFinished before we check the running state in
+	// order to avoid a race condition.
+	QEventLoop loop;
+	connect(p, SIGNAL(processFinished()), &loop, SLOT(quit()));
+	if (p->isRunning()) {
+		processWaitedFor = p;
+		emit buildRunning(true);
+		loop.exec(); //exec will delay execution until the signal has arrived
+		emit buildRunning(false);
+		processWaitedFor = nullptr;
 	}
-	QApplication::restoreOverrideCursor();
-	bool result = processWaitedFor;
-    processWaitedFor = nullptr;
-	m_stopBuildAction->setEnabled(false);
-	return result;
+	return true;
 }
 
 bool BuildManager::waitingForProcess() const
@@ -1761,7 +1795,7 @@ void BuildManager::preview(const QString &preamble, const PreviewSource &source,
 				QFileInfo fi(*tf);
 				preambleFormatFile = fi.completeBaseName();
 				previewFileNames.append(fi.absoluteFilePath());
-                ProcessX *p = nullptr;
+				ProcessX *p = nullptr;
 				if (dvi2pngMode == DPM_EMBEDDED_PDF) {
 					p = newProcessInternal(QString("%1 -interaction=nonstopmode -ini \"&pdflatex %3 \\dump\"").arg(getCommandInfo(CMD_PDFLATEX).getProgramName()).arg(preambleFormatFile), tf->fileName()); //no delete! goes automatically
 				} else {
@@ -1941,9 +1975,16 @@ void BuildManager::setAllCommands(const CommandMapping &cmds, const QStringList
 QString BuildManager::guessCompilerFromProgramMagicComment(const QString &program)
 {
 	if (program == "latex") return BuildManager::CMD_LATEX;
-	else if (program == "pdflatex") return BuildManager::CMD_PDFLATEX;
-	else if (program == "xelatex") return BuildManager::CMD_XELATEX;
-	else if (program == "luatex" || program == "lualatex") return BuildManager::CMD_LUALATEX;
+    if (program == "pdflatex") return BuildManager::CMD_PDFLATEX;
+    if (program == "xelatex") return BuildManager::CMD_XELATEX;
+    if (program == "luatex" || program == "lualatex") return BuildManager::CMD_LUALATEX;
+    if (program.startsWith("user")){
+        bool user;
+        QString cmd=getCommandLine(program,&user);
+        if(user){
+            return cmd;
+        }
+    }
 	return QString();
 
 }
@@ -1953,7 +1994,8 @@ QString BuildManager::guessCompilerFromProgramMagicComment(const QString &progra
  */
 QString BuildManager::guessViewerFromProgramMagicComment(const QString &program)
 {
-	if (program == "latex") return BuildManager::CMD_VIEW_DVI;
+	if (program == "latex")
+		return BuildManager::CMD_VIEW_DVI;
 	else if (program == "pdflatex" || program == "xelatex" || program == "luatex" || program == "lualatex") {
 		return CMD_VIEW_PDF;
 	}
@@ -1962,17 +2004,18 @@ QString BuildManager::guessViewerFromProgramMagicComment(const QString &program)
 
 void BuildManager::singleInstanceCompleted(int status)
 {
-	Q_UNUSED(status);
+	Q_UNUSED(status)
 	QObject *s = sender();
 	REQUIRE(s);
 	for (QMap<QString, ProcessX *>::iterator it = runningCommands.begin(), end = runningCommands.end(); it != end;)
-		if (it.value() == s) it = runningCommands.erase(it);
+		if (it.value() == s)
+			it = runningCommands.erase(it);
 		else ++it;
 }
 
 void BuildManager::preamblePrecompileCompleted(int status)
 {
-	Q_UNUSED(status);
+	Q_UNUSED(status)
 	QProcess *p = qobject_cast<QProcess *>(sender());
 	REQUIRE(p);
 	if (p->exitCode() != 0 || p->exitStatus() != QProcess::NormalExit) {
@@ -1985,12 +2028,12 @@ void BuildManager::preamblePrecompileCompleted(int status)
 //now either dvips or dvipng is necessary if not already running
 void BuildManager::latexPreviewCompleted(int status)
 {
-	Q_UNUSED(status);
+	Q_UNUSED(status)
 	if (dvi2pngMode == DPM_DVIPNG) {
 		ProcessX *p1 = qobject_cast<ProcessX *> (sender());
 		if (!p1) return;
 		// dvi -> png
-        ProcessX *p2 = firstProcessOfDirectExpansion(CMD_DVIPNG, p1->getFile(),QFileInfo(),0,true);
+		ProcessX *p2 = firstProcessOfDirectExpansion(CMD_DVIPNG, p1->getFile(),QFileInfo(),0,true);
 		if (!p2) return; //dvipng does not work
 		//REQUIRE(p2);
 		if (!p1->overrideEnvironment().isEmpty()) p2->setOverrideEnvironment(p1->overrideEnvironment());
@@ -2001,7 +2044,7 @@ void BuildManager::latexPreviewCompleted(int status)
 		ProcessX *p1 = qobject_cast<ProcessX *> (sender());
 		if (!p1) return;
 		// dvi -> ps
-        ProcessX *p2 = firstProcessOfDirectExpansion("txs:///dvips/[-E]", p1->getFile(),QFileInfo(),0,true);
+		ProcessX *p2 = firstProcessOfDirectExpansion("txs:///dvips/[-E]", p1->getFile(),QFileInfo(),0,true);
 		if (!p2) return; //dvips does not work
 		//REQUIRE(p2);
 		if (!p1->overrideEnvironment().isEmpty()) p2->setOverrideEnvironment(p1->overrideEnvironment());
@@ -2027,29 +2070,29 @@ void BuildManager::latexPreviewCompleted(int status)
 //dvi to ps conversion is finished, call ghostscript to make a useable png from it
 void BuildManager::dvi2psPreviewCompleted(int status)
 {
-	Q_UNUSED(status);
+	Q_UNUSED(status)
 	ProcessX *p2 = qobject_cast<ProcessX *> (sender());
 	if (!p2) return;
 	// ps -> png, ghostscript is quite, safe, will create 24-bit png
 	QString filePs = parseExtendedCommandLine("?am.ps", p2->getFile()).first();
-    ProcessX *p3 = firstProcessOfDirectExpansion("txs:///gs/[-q][-dSAFER][-dBATCH][-dNOPAUSE][-sDEVICE=png16m][-dEPSCrop][-sOutputFile=\"?am)1.png\"]", filePs,QFileInfo(),0,true);
-    if (!p3) return; //gs does not work
+	ProcessX *p3 = firstProcessOfDirectExpansion("txs:///gs/[-q][-dSAFER][-dBATCH][-dNOPAUSE][-sDEVICE=png16m][-dEPSCrop][-sOutputFile=\"?am)1.png\"]", filePs,QFileInfo(),0,true);
+	if (!p3) return; //gs does not work
 	if (!p2->overrideEnvironment().isEmpty()) p3->setOverrideEnvironment(p2->overrideEnvironment());
 	connect(p3, SIGNAL(finished(int)), this, SLOT(conversionPreviewCompleted(int)));
 	p3->startCommand();
 }
 void BuildManager::PreviewLatexCompleted(int status){
-    if(status!=0){
-        // latex compile failed, kill dvipng
-        ProcessX *p1 = qobject_cast<ProcessX *> (sender());
-        ProcessX *p2=p1->property("proc").value<ProcessX *>();
-        p2->terminate();
-    }
+	if(status!=0){
+		// latex compile failed, kill dvipng
+		ProcessX *p1 = qobject_cast<ProcessX *> (sender());
+		ProcessX *p2=p1->property("proc").value<ProcessX *>();
+		p2->terminate();
+	}
 }
 
 void BuildManager::conversionPreviewCompleted(int status)
 {
-	Q_UNUSED(status);
+	Q_UNUSED(status)
 	ProcessX *p2 = qobject_cast<ProcessX *> (sender());
 	if (!p2) return;
 	// put image in preview
@@ -2091,10 +2134,10 @@ bool BuildManager::testAndRunInternalCommand(const QString &cmd, const QFileInfo
 	int space = cmd.indexOf(' ');
 	QString cmdId, options;
 	if (space == -1 ) cmdId = cmd;
-    else {
-        cmdId = cmd.left(space);
-        options = cmd.mid(space + 1);
-    }
+	else {
+		cmdId = cmd.left(space);
+		options = cmd.mid(space + 1);
+	}
 	if (internalCommands.contains(cmdId)) {
 		emit runInternalCommand(cmdId, mainFile, options);
 		return true;
@@ -2102,30 +2145,66 @@ bool BuildManager::testAndRunInternalCommand(const QString &cmd, const QFileInfo
 	return false;
 }
 
-QString BuildManager::findFile(const QString &defaultName, const QStringList &searchPaths)
+QString BuildManager::findFile(const QString &defaultName, const QStringList &searchPaths, bool mostRecent)
 {
 	//TODO: merge with findResourceFile
 	QFileInfo base(defaultName);
-	if (base.exists()) return defaultName;
-	if (searchPaths.isEmpty()) return "";
+	QFileInfo* mr = nullptr;
+	if (base.exists()) {
+		if (mostRecent)
+			mr = new QFileInfo(base);
+		else
+			return defaultName;
+	}
 
 	foreach (QString p, searchPaths) {
+        QFileInfo fi;
 		if (p.startsWith('/') || p.startsWith("\\\\") || (p.length() > 2 && p[1] == ':' && (p[2] == '\\' || p[2] == '/'))) {
-			QFileInfo fi(QDir(p), base.fileName());
-			if (fi.exists()) return fi.absoluteFilePath();
+			fi = QFileInfo(QDir(p), base.fileName());
 		} else {
 			// ?? seems a bit weird: if p is not an absolute path, then interpret p as directory
 			// e.g. default = /my/filename.tex
-			//      p = foo
+			//	  p = foo
 			// -->  /my/foo/filename.tex
 			// TODO: do we want/use this anywere or can it be removed?
 			QString absPath = base.absolutePath() + "/";
 			QString baseName = "/" + base.fileName();
-			QFileInfo fi(absPath + p + baseName);
-			if (fi.exists()) return fi.absoluteFilePath();
+			fi = QFileInfo(absPath + p + baseName);
+		}
+		if (fi.exists()) {
+			if (mostRecent) {
+				if (mr == nullptr || mr->lastModified() < fi.lastModified()) {
+					if (mr != nullptr)
+						delete mr;
+					mr = new QFileInfo(fi);
+				}
+			} else
+				return fi.absoluteFilePath();
 		}
 	}
-	return "";
+	if (mostRecent && mr != nullptr) {
+		QString result = mr->absoluteFilePath();
+		delete mr;
+		return result;
+	} else {
+		return "";
+	}
+}
+
+QString BuildManager::findCompiledFile(const QString &compiledFilename, const QFileInfo &mainFile)
+{
+	QStringList searchPaths;
+	QString foundPathname;
+
+	searchPaths << mainFile.absolutePath();
+	searchPaths << splitPaths(resolvePaths(additionalPdfPaths));
+	foundPathname = findFile(compiledFilename, searchPaths, true);
+	if (foundPathname == "") {
+		// If searched filename is relative prepend the mainFile directory
+		// so PDF viewer shows a reasonable error message
+		foundPathname = QFileInfo(mainFile.absoluteDir(), compiledFilename).absoluteFilePath();
+	}
+	return (foundPathname);
 }
 
 void BuildManager::removePreviewFiles(QString elem)
@@ -2302,18 +2381,8 @@ void ProcessX::startCommand()
 	if (stderrEnabled)
 		connect(this, SIGNAL(readyReadStandardError()), this, SLOT(readFromStandardError()));
 
-	//qDebug() << workingDirectory();
-	//qDebug() << cmd;
-	QByteArray path = qgetenv("PATH");
-#ifdef Q_OS_OSX
-	QString basePath = getEnvironmentPath();
-    qputenv("PATH", path + getPathListSeparator().toLatin1() + BuildManager::resolvePaths(BuildManager::additionalSearchPaths).toUtf8() + getPathListSeparator().toLatin1() + basePath.toUtf8());
-	// needed for searching the executable in the additional paths see https://bugreports.qt-project.org/browse/QTBUG-18387
-#else
-	qputenv("PATH", path + getPathListSeparator().toLatin1() + BuildManager::resolvePaths(BuildManager::additionalSearchPaths).toUtf8()); // needed for searching the executable in the additional paths see https://bugreports.qt-project.org/browse/QTBUG-18387
-#endif
-	QProcess::start(cmd);
-	qputenv("PATH", path); // restore
+	ExecProgram execProgram(cmd, BuildManager::resolvePaths(BuildManager::additionalSearchPaths));
+	execProgram.execAndNoWait(*this);
 
 	if (error() == FailedToStart || error() == Crashed)
 		isStarted = ended = true; //prevent call of waitForStarted, if it failed to start (see QTBUG-33021)
@@ -2451,6 +2520,7 @@ void ProcessX::onFinished(int error)
 		readFromStandardError();
 	}
 	ended = true;
+	emit processFinished();
 }
 
 #ifdef PROFILE_PROCESSES
