@@ -40,9 +40,8 @@
 */
 
 QDocumentSearch::QDocumentSearch(QEditor *e, const QString& f, Options opt, const QString& r)
- : m_option(opt), m_string(f),  m_editor(e), m_replaced(0), m_replaceDeltaLength(0)
+  : m_option(opt), m_string(f), m_replace(r), m_editor(e), m_replaced(0), m_replaceDeltaLength(0)
 {
-	m_replace=r;
 	connectToEditor();
 }
 void QDocumentSearch::connectToEditor(){
@@ -56,7 +55,7 @@ void QDocumentSearch::connectToEditor(){
 
 QDocumentSearch::~QDocumentSearch()
 {
-        clearMatches();
+	clearMatches();
 }
 
 QDocumentSearch::Options QDocumentSearch::options() const
@@ -83,7 +82,7 @@ void QDocumentSearch::searchMatches(const QDocumentCursor& subHighlightScope, bo
 		return;
 	}
 
-	QDocumentCursor hscope = subHighlightScope; 
+	QDocumentCursor hscope = subHighlightScope;
 	if (!hscope.isValid() || !hscope.hasSelection() || hscope.document()!=d){
 		if (m_scope.isValid() && m_scope.hasSelection() && m_scope.document()==d) hscope=m_scope;
 		else {
@@ -119,7 +118,7 @@ void QDocumentSearch::searchMatches(const QDocumentCursor& subHighlightScope, bo
 				hscope = QDocumentCursor(hss, m_searchedScope.selectionStart());
 				m_searchedScope.setAnchorLineNumber(hss.lineNumber());
 				m_searchedScope.setAnchorColumnNumber(hss.columnNumber());
-			} else m_searchedScope = QDocumentCursor(hss, hse);;
+			} else m_searchedScope = QDocumentCursor(hss, hse);
 		} else if (!startInOld || !endInOld){
 			if (startInOld) {
 				m_searchedScope.setLineNumber(hse.lineNumber());
@@ -127,7 +126,7 @@ void QDocumentSearch::searchMatches(const QDocumentCursor& subHighlightScope, bo
 			} else if (endInOld) {
 				m_searchedScope.setAnchorLineNumber(hss.lineNumber());
 				m_searchedScope.setAnchorColumnNumber(hss.columnNumber());
-			} else m_searchedScope = QDocumentCursor(hss, hse);;
+			} else m_searchedScope = QDocumentCursor(hss, hse);
 		}
 	} else m_searchedScope = QDocumentCursor(hscope.selectionStart(), hscope.selectionEnd());
 
@@ -157,8 +156,15 @@ void QDocumentSearch::searchMatches(const QDocumentCursor& subHighlightScope, bo
 		l.setFlag(QDocumentLine::FormatsApplied, false);
 
 		const QString &s = boundaries.endLine != ln ? l.text() : l.text().left(boundaries.end);
-
+#if QT_VERSION > 0x050500
+		m_match=m_regularExpression.match(s, hc.columnNumber());
+		int column = m_match.capturedStart();
+		int length = m_match.capturedLength();
+#else
 		int column=m_regexp.indexIn(s, hc.columnNumber());
+		int length=m_regexp.matchedLength();
+#endif
+
 		/*
 		qDebug("searching %s in %s => %i",
 				qPrintable(m_regexp.pattern()),
@@ -168,47 +174,58 @@ void QDocumentSearch::searchMatches(const QDocumentCursor& subHighlightScope, bo
 		
 		if ( column != -1 && (column >= hc.columnNumber() ) )
 		{
-			if (!m_regexp.matchedLength())
+			if (length==0)
 				hc.setColumnNumber(column+1); //empty (e.g. a* regexp)
 			else {
 				hc.setColumnNumber(column);
-				hc.setColumnNumber(column + m_regexp.matchedLength(), QDocumentCursor::KeepAnchor);
-											
-				hc.line().addOverlay(QFormatRange(hc.anchorColumnNumber(), hc.columnNumber() - hc.anchorColumnNumber(), sid));
-				m_highlights.insert(l.handle());
+				hc.setColumnNumber(column + length, QDocumentCursor::KeepAnchor);
+
+				if (isAcceptedFormat(l.getCachedFormatAt(column))){
+					// add filtered or all
+					hc.line().addOverlay(QFormatRange(hc.anchorColumnNumber(), hc.columnNumber() - hc.anchorColumnNumber(), sid));
+					m_highlights.insert(l.handle());
+				}
 			}
 		} else hc.movePosition(1, QDocumentCursor::NextBlock, QDocumentCursor::ThroughFolding);
 	}
-    int begLine=0;
-    int endLine=d->lines();
-    int offset=0;
-    int endOffset=-1;
-    if (m_scope.isValid() && m_scope.hasSelection()){
-        QDocumentSelection boundaries=m_scope.selection();
-        begLine=boundaries.startLine;
-        endLine=boundaries.endLine+1;
-        offset=boundaries.start;
-        endOffset=boundaries.end;
-    }
-    if(clearSelection){
-        // don't run again when only visibleLines is changed
-        // remove old marks first
-        m_editor->removeMark("search");
-        bool needsUpdate=false;
-        for(int i=begLine;i<endLine;i++){
-            QString txt=d->line(i).text();
-            if((endOffset>=0)&&(i+1==endLine)){
-                txt=txt.left(endOffset);
-            }
-            if(m_regexp.indexIn(txt,offset)>-1){
-                m_editor->addMarkDelayed(i,Qt::darkYellow,"search");
-                needsUpdate=true;
-            }
-            offset=0;
-        }
-        if(needsUpdate)
-            m_editor->paintMarks();
-    }
+	int begLine=0;
+	int endLine=d->lines();
+	int offset=0;
+	int endOffset=-1;
+	if (m_scope.isValid() && m_scope.hasSelection()){
+		QDocumentSelection boundaries=m_scope.selection();
+		begLine=boundaries.startLine;
+		endLine=boundaries.endLine+1;
+		offset=boundaries.start;
+		endOffset=boundaries.end;
+	}
+	if(clearSelection){
+		// don't run again when only visibleLines is changed
+		// remove old marks first
+		m_editor->removeMark("search");
+		bool needsUpdate=false;
+		for(int i=begLine;i<endLine;i++){
+			QString txt=d->line(i).text();
+			if((endOffset>=0)&&(i+1==endLine)){
+				txt=txt.left(endOffset);
+			}
+#if QT_VERSION > 0x050500
+			QRegularExpressionMatch match=m_regularExpression.match(txt,offset);
+			if(match.hasMatch()){
+				m_editor->addMarkDelayed(i,Qt::darkYellow,"search");
+				needsUpdate=true;
+			}
+#else
+			if(m_regexp.indexIn(txt,offset)>-1){
+				m_editor->addMarkDelayed(i,Qt::darkYellow,"search");
+				needsUpdate=true;
+			}
+#endif
+			offset=0;
+		}
+		if(needsUpdate)
+			m_editor->paintMarks();
+	}
 	m_editor->viewport()->update();
 }
 
@@ -229,10 +246,10 @@ void QDocumentSearch::clearMatches()
 	foreach (QDocumentLineHandle* h, m_highlights)
 		QDocumentLine(h).clearOverlays(sid);
 
-    m_editor->removeMark("search");
-    m_editor->removeMark("replace");
+	m_editor->removeMark("search");
+	m_editor->removeMark("replace");
 	m_highlights.clear();
-    m_newReplacementOverlays.clear();
+	m_newReplacementOverlays.clear();
 	m_searchedScope = QDocumentCursor();
 	//qDebug("clearing matches");
 
@@ -251,28 +268,42 @@ void QDocumentSearch::clearReplacements(){
 
 	foreach (QDocumentLineHandle* l, m_highlightedReplacements)
 		QDocumentLine(l).clearOverlays(sid);
-//	m_editor->viewport()->update();
+	//	m_editor->viewport()->update();
 
 	m_highlightedReplacements.clear();
 }
 
 /*
-  returns the document the search should be performed
-  */
+	returns the document the search should be performed
+	*/
 QDocument* QDocumentSearch::currentDocument(){
 	return m_editor ? m_editor->document() : m_cursor.document();
 }
 
 /*
-  returns a regexp which fully describes the search text (including flags like casesensitivity, wholewords,...)
-  */
+	returns a regexp which fully describes the search text (including flags like casesensitivity, wholewords,...)
+	*/
 void QDocumentSearch::recreateRegExp(){
 	Qt::CaseSensitivity cs = hasOption(CaseSensitive)
-								?
-									Qt::CaseSensitive
-								:
-									Qt::CaseInsensitive;
-	
+	    ?
+	      Qt::CaseSensitive
+	    :
+	      Qt::CaseInsensitive;
+
+#if QT_VERSION >= 0x050500
+	QRegularExpression::PatternOption patternOption= cs==Qt::CaseInsensitive ? QRegularExpression::CaseInsensitiveOption : QRegularExpression::NoPatternOption ;
+	if ( hasOption(RegExp) )
+	{
+
+		m_regularExpression = QRegularExpression(m_string, patternOption);
+	} else if ( hasOption(WholeWords) ) {
+		//todo: screw this? it prevents searching of "world!" and similar things
+		//(qtextdocument just checks the surrounding character when searching for whole words, this would also allow wholewords|regexp search)
+		m_regularExpression = QRegularExpression( QString("\\b%1\\b").arg(QRegularExpression::escape(m_string)), patternOption );
+	} else {
+		m_regularExpression = QRegularExpression(QRegularExpression::escape(m_string), patternOption);
+	}
+#else
 	if ( hasOption(RegExp) )
 	{
 		m_regexp = QRegExp(m_string, cs, QRegExp::RegExp);
@@ -280,13 +311,15 @@ void QDocumentSearch::recreateRegExp(){
 		//todo: screw this? it prevents searching of "world!" and similar things
 		//(qtextdocument just checks the surrounding character when searching for whole words, this would also allow wholewords|regexp search)
 		m_regexp = QRegExp(
-						QString("\\b%1\\b").arg(QRegExp::escape(m_string)),
-						cs,
-						QRegExp::RegExp
-					);
+		      QString("\\b%1\\b").arg(QRegExp::escape(m_string)),
+		      cs,
+		      QRegExp::RegExp
+		      );
 	} else {
 		m_regexp = QRegExp(m_string, cs, QRegExp::FixedString);
 	}
+	m_regexp.setMinimal( m_option & QDocumentSearch::NonGreedy); // allow greedy or non-greedy capture
+#endif
 }
 
 
@@ -303,7 +336,7 @@ QString QDocumentSearch::searchText() const
 */
 void QDocumentSearch::setSearchText(const QString& f)
 {
-	if (m_string == f) 
+	if (m_string == f)
 		return;
 	
 	m_string = f;
@@ -355,16 +388,16 @@ void QDocumentSearch::setOption(Option opt, bool on)
 		else if ( on )
 			searchMatches();
 	} else if (
-					(m_option & QDocumentSearch::HighlightAll)
-				&&
-					(
-						(opt & QDocumentSearch::RegExp)
-					||
-						(opt & QDocumentSearch::WholeWords)
-					||
-						(opt & QDocumentSearch::CaseSensitive)
-					)
-			)
+	           (m_option & QDocumentSearch::HighlightAll)
+	           &&
+	           (
+	             (opt & QDocumentSearch::RegExp)
+	             ||
+	             (opt & QDocumentSearch::WholeWords)
+	             ||
+	             (opt & QDocumentSearch::CaseSensitive)
+	             )
+	           )
 	{
 		// matches may have become invalid : update them
 		searchMatches();
@@ -375,7 +408,28 @@ void QDocumentSearch::setOption(Option opt, bool on)
 
 void QDocumentSearch::setOptions(Options options){
 	for (int i=0;i<8;i++)
-		setOption((Option)(1<<i), options & (1<<i));
+		setOption(static_cast<Option>(1<<i), options & (1<<i));
+}
+void QDocumentSearch::setFilteredFormats(QList<int> ids, bool inverted){
+	m_filteredIds=ids;
+	m_filteredIdsInverted = inverted;
+	// update search
+	if (m_option & QDocumentSearch::HighlightAll){
+		// matches may have become invalid : update them
+		searchMatches();
+		visibleLinesChanged();
+	}
+}
+
+QList<int> QDocumentSearch::getFilteredFormats() const
+{
+	return m_filteredIds;
+}
+
+bool QDocumentSearch::isAcceptedFormat(int formatIds) const {
+	if (m_filteredIds.isEmpty()) return true;
+	bool contained =  m_filteredIds.contains(formatIds&255)|| m_filteredIds.contains((formatIds>>8)&255)|| m_filteredIds.contains((formatIds>>16)&255);
+	return contained != m_filteredIdsInverted;
 }
 
 
@@ -400,7 +454,7 @@ void QDocumentSearch::setReplaceText(const QString& r)
 static QString escapeCpp(const QString& s)
 {
 	QString es;
-//TODO: numbers (e.g. \xA6)
+	//TODO: numbers (e.g. \xA6)
 	for ( int i = 0; i < s.count(); ++i )
 	{
 		if ( (s.at(i) == '\\') && ((i + 1) < s.count()) )
@@ -417,7 +471,7 @@ static QString escapeCpp(const QString& s)
 				es += '\r';
 			//else if ( c == '0' )
 			//	es += '\0';
-			else es += '\\', es += c;
+			else es += '\\' + c;
 
 		} else {
 			es += s.at(i);
@@ -438,13 +492,20 @@ QString QDocumentSearch::replaceTextExpanded() const
 	QString replacement = hasOption(EscapeSeq)?escapeCpp(m_replace):m_replace;
 
 	if (hasOption(RegExp))
-#if QT_VERSION<0x040600
-	   for ( int i = m_regexp.numCaptures(); i >= 0; --i )
+#if QT_VERSION>0x050500
+		for( int i=m_match.lastCapturedIndex();i >=0; --i )
+			replacement.replace(QString("\\") + QString::number(i),
+			                    m_match.captured(i));
 #else
-	   for ( int i = m_regexp.captureCount(); i >= 0; --i )
+#if QT_VERSION<0x040600
+		for ( int i = m_regexp.numCaptures(); i >= 0; --i )
+#else
+		for ( int i = m_regexp.captureCount(); i >= 0; --i )
 #endif
 			replacement.replace(QString("\\") + QString::number(i),
-								m_regexp.cap(i));
+			                    m_regexp.cap(i));
+#endif
+
 	return replacement;
 }
 
@@ -515,7 +576,7 @@ void QDocumentSearch::setScope(const QDocumentCursor& c)
 	\param backward whether to go backward or forward
 	\param all if true, the whole document will be searched first, all matches recorded and available for further navigation
 	\param again if a search match is selected it will be replaced, than a normal search (no replace) will be performed
-	\return The number of found occurences
+		\return The number of found occurrences
 
 	\note The search will start at the first character left/right from the selected text
 */
@@ -524,7 +585,7 @@ int QDocumentSearch::next(bool backward, bool all, bool again, bool allowWrapAro
 	if ( m_string.isEmpty() )
 		return 0;
 
-    if ( overrideScope && !overrideScope->isValid() ) overrideScope = nullptr;
+	if ( overrideScope && !overrideScope->isValid() ) overrideScope = nullptr;
 	
 	const QDocumentCursor& scope = overrideScope ? *overrideScope : m_scope;
 	
@@ -544,7 +605,7 @@ int QDocumentSearch::next(bool backward, bool all, bool again, bool allowWrapAro
 				m_cursor.movePosition(1, QDocumentCursor::End);
 			
 		} else {
-            QMessageBox::warning(nullptr, nullptr, "Unable to perform search operation");
+			QMessageBox::warning(nullptr, nullptr, "Unable to perform search operation");
 		}
 	}
 
@@ -557,9 +618,32 @@ int QDocumentSearch::next(bool backward, bool all, bool again, bool allowWrapAro
 	// replace
 	if (hasOption(Replace) && again && !all) {
 		bool replaceSelectedText = false;
+#if QT_VERSION > 0x050500
+		if (m_match.hasMatch() && m_match.captured()==m_cursor.selectedText())  {
+			replaceSelectedText = true;
+		} else if (m_regularExpression.pattern().contains("(?=") || m_regularExpression.pattern().contains("(?!")) {
+			// special handling for lookahead: The selected text is not enough to match the regexp
+			// because the lookahead context is missing. Therefore we have to find matches to the
+			// whole line until we find the original selection. Only then, we know that the original
+			// selection is match and should be replaced.
+			int start = 0;
+			while (true) {
+				QRegularExpressionMatch match = m_regularExpression.match(m_cursor.line().text(), start);
+				start=match.capturedStart();
+				if (start < 0)
+					break;
+				int end = start + match.capturedLength();
+				if ((start == m_cursor.startColumnNumber() && end == m_cursor.endColumnNumber()) ||
+				    (end == m_cursor.startColumnNumber() && start == m_cursor.endColumnNumber())) {
+					replaceSelectedText = true;
+				}
+				start = end;
+			}
+		}
+#else
 		if (m_regexp.exactMatch(m_cursor.selectedText()))  {
 			replaceSelectedText = true;
-		} else if (m_regexp.pattern().contains("(?=") or m_regexp.pattern().contains("(?!")) {
+		} else if (m_regexp.pattern().contains("(?=") || m_regexp.pattern().contains("(?!")) {
 			// special handling for lookahead: The selected text is not enough to match the regexp
 			// because the lookahead context is missing. Therefore we have to find matches to the
 			// whole line until we find the original selection. Only then, we know that the original
@@ -577,6 +661,7 @@ int QDocumentSearch::next(bool backward, bool all, bool again, bool allowWrapAro
 				start = end;
 			}
 		}
+#endif
 		if (replaceSelectedText) {
 			replaceCursorText(backward);
 			updateReplacementOverlays();
@@ -591,7 +676,7 @@ int QDocumentSearch::next(bool backward, bool all, bool again, bool allowWrapAro
 	//ensure that the current selection isn't searched
 	if ( m_cursor.hasSelection() ) {
 		if (m_cursor.selectionStart() == scope.selectionStart() &&
-			m_cursor.selectionEnd() == scope.selectionEnd()) {
+		    m_cursor.selectionEnd() == scope.selectionEnd()) {
 			//search whole scope
 			if (backward) m_cursor=scope.selectionEnd();
 			else m_cursor=scope.selectionStart();
@@ -609,11 +694,11 @@ int QDocumentSearch::next(bool backward, bool all, bool again, bool allowWrapAro
 	// condition only to avoid debug messages...
 	if ( bounded ) {
 		boundaries = scope.selection();
-	
+
 		//moves the cursor in the search scope if it isn't there, but directly in front of the selection (only possible if there actually is a selection)
 		if (   ( backward ? m_cursor.atStart() : m_cursor.atEnd() )                              //absEnd
-		    || ( scope.isValid() && scope.hasSelection() && !scope.isWithinSelection(m_cursor) )
-		   ) {
+		       || ( scope.isValid() && scope.hasSelection() && !scope.isWithinSelection(m_cursor) )
+		       ) {
 			if ( !backward && m_cursor < scope.selectionStart() ) {
 				m_cursor = scope.selectionStart();
 			} else {
@@ -627,7 +712,7 @@ int QDocumentSearch::next(bool backward, bool all, bool again, bool allowWrapAro
 	
 	QDocumentCursor::MoveOperation move;
 	
-	move = backward ? QDocumentCursor::PreviousBlock : QDocumentCursor::NextBlock;	
+	move = backward ? QDocumentCursor::PreviousBlock : QDocumentCursor::NextBlock;
 	
 	int foundCount = 0;
 
@@ -638,10 +723,10 @@ int QDocumentSearch::next(bool backward, bool all, bool again, bool allowWrapAro
 	
 	m_cursor.setColumnMemory(false);
 	QDocumentCursor lastSelection;
-	while ( !( 
-	             ( backward ? m_cursor.atStart() : m_cursor.atEnd() )                              //absEnd
+	while ( !(
+	          ( backward ? m_cursor.atStart() : m_cursor.atEnd() )                              //absEnd
 	          || ( scope.isValid() && scope.hasSelection() && !scope.isWithinSelection(m_cursor) )
-	      ) )
+	          ) )
 	{
 		if ( backward && !m_cursor.columnNumber() )
 		{
@@ -655,6 +740,11 @@ int QDocumentSearch::next(bool backward, bool all, bool again, bool allowWrapAro
 		int coloffset = 0;
 		QString s = l.text();
 
+		if ( bounded ) {
+			// update boundaries as scope is changed when changing text
+			boundaries = scope.selection();
+		}
+
 		if ( backward )
 		{
 			if ( bounded && (boundaries.startLine == ln) )
@@ -671,18 +761,49 @@ int QDocumentSearch::next(bool backward, bool all, bool again, bool allowWrapAro
 		}
 		
 		int column;
+		int length;
+#if QT_VERSION > 0x050500
+		if (backward) {
+			column=s.lastIndexOf(m_regularExpression,m_cursor.columnNumber()-coloffset,&m_match);
+			length=m_match.capturedLength();
+		}else {
+			m_match=m_regularExpression.match(s, m_cursor.columnNumber());
+			column=m_match.capturedStart();
+			length=m_match.capturedLength();
+		}
+
+
+#else
 		if (backward) column=m_regexp.lastIndexIn(s,m_cursor.columnNumber()-coloffset);
 		else column=m_regexp.indexIn(s, m_cursor.columnNumber());
-        /*
+		length=m_regexp.matchedLength();
+#endif
+
+		QDocumentCursor::MoveOperation nextMove = move;
+
+		bool matches = column != -1 && (backward || column >= m_cursor.columnNumber() );
+
+		if(backward && hasOption(RegExp) && m_string.endsWith('$') && s.length()<l.length()){
+			matches = false; // force miss as regexp $ is only valid on unchanged line
+		}
+
+		if(matches){
+			if(!isAcceptedFormat(l.getCachedFormatAt(column))){
+				//filter out matches that don't fulfill formatting e.g. math-env
+				nextMove = backward ? QDocumentCursor::PreviousCharacter : QDocumentCursor::NextCharacter;
+				matches = false;
+			}
+		}
+		/*
 		qDebug("searching %s in %s from %i => %i",
 				qPrintable(m_regexp.pattern()),
-				qPrintable(s), m_cursor.columnNumber(), 
+				qPrintable(s), m_cursor.columnNumber(),
 				column);
 		//*/
 
-		if ( column != -1 && (backward || column >= m_cursor.columnNumber() ) )
+		if ( matches )
 		{
-			if (!m_regexp.matchedLength()){
+			if (!length){
 				//empty (e.g. a* regexp)
 				if (backward) m_cursor.setColumnNumber(column-1);
 				else m_cursor.setColumnNumber(column+1);
@@ -691,11 +812,11 @@ int QDocumentSearch::next(bool backward, bool all, bool again, bool allowWrapAro
 				
 				if ( backward )
 				{
-					m_cursor.setColumnNumber(column + m_regexp.matchedLength());
+					m_cursor.setColumnNumber(column + length);
 					m_cursor.setColumnNumber(column, QDocumentCursor::KeepAnchor);
 				} else {
 					m_cursor.setColumnNumber(column);
-					m_cursor.setColumnNumber(column + m_regexp.matchedLength(), QDocumentCursor::KeepAnchor);
+					m_cursor.setColumnNumber(column + length, QDocumentCursor::KeepAnchor);
 				}
 				
 				if ( m_editor && !hasOption(Silent)) {
@@ -719,13 +840,13 @@ int QDocumentSearch::next(bool backward, bool all, bool again, bool allowWrapAro
 					{
 						QMessageBox::StandardButtons buttons=QMessageBox::Yes | QMessageBox::No;
 						if (all) buttons|=QMessageBox::Cancel;
-											int ret = QMessageBox::question(m_editor, tr("Replacement prompt"),
-											tr("Shall it be replaced?"),
-											buttons,
-											QMessageBox::Yes);
+						int ret = QMessageBox::question(m_editor, tr("Replacement prompt"),
+						                                tr("Shall it be replaced?"),
+						                                buttons,
+						                                QMessageBox::Yes);
 						rep=ret==QMessageBox::Yes;
 						if (ret==QMessageBox::Cancel) {
-							QMessageBox::information(m_editor,tr("Replacing Canceled"),tr("%1 (of %2 found so far) occurences have been replaced").arg(replaceCount).arg(foundCount),QMessageBox::Ok);
+							QMessageBox::information(m_editor,tr("Replacing Canceled"),tr("%1 (of %2 found so far) occurrences have been replaced").arg(replaceCount).arg(foundCount),QMessageBox::Ok);
 							return foundCount;
 						}
 					}
@@ -735,14 +856,15 @@ int QDocumentSearch::next(bool backward, bool all, bool again, bool allowWrapAro
 						if (!all) updateReplacementOverlays();
 						replaceCount++;
 					}
-				} 
+				}
 				
 				
 				if ( !all )
 					break;
 			}
-		} else 
-			m_cursor.movePosition(1, move, QDocumentCursor::ThroughFolding);
+		} else {
+			m_cursor.movePosition(1, nextMove, QDocumentCursor::ThroughFolding);
+		}
 	}
 	if ( all && replaceCount )
 		updateReplacementOverlays();
@@ -750,37 +872,37 @@ int QDocumentSearch::next(bool backward, bool all, bool again, bool allowWrapAro
 		m_editor->document()->endDelayedUpdateBlock();
 		m_editor->document()->endMacro();
 		if (!hasOption(Silent)){
-            //m_editor->setCursor(lastSelection, false);
+			//m_editor->setCursor(lastSelection, false);
 			m_editor->ensureCursorVisible(QEditor::Navigation);
 		}
 	}
-		
+
 	if ( !foundCount && allowWrapAround)
 	{
 		m_cursor = QDocumentCursor();
-			
+
 		int ret = QMessageBox::Yes; //different to base qce2.2, where it defaults to ::no if silent
 		if ( !hasOption(Silent) /*&& hasOption(Prompt), to ask or not to ask that is the question; if it doesn't ask it fails silently if no match exists*/){
 			QString message;
 			if (backward) {
 				message = tr(
-							  "Start of scope reached with no match.\n"
-							  "Restart from the end?"
-						  );
+				      "Start of scope reached with no match.\n"
+				      "Restart from the end?"
+				      );
 			} else {
 				message = tr(
-							  "End of scope reached with no match.\n"
-							  "Restart from the beginning?"
-						  );
+				      "End of scope reached with no match.\n"
+				      "Restart from the beginning?"
+				      );
 			}
 			ret=QMessageBox::question(
-							m_editor,
-							tr("Failure"),
-							message,
-							QMessageBox::Yes
-							| QMessageBox::No,
-							QMessageBox::Yes
-						);
+			      m_editor,
+			      tr("Failure"),
+			      message,
+			      QMessageBox::Yes
+			      | QMessageBox::No,
+			      QMessageBox::Yes
+			      );
 		}
 		if ( ret == QMessageBox::Yes )
 			return next(backward, all, again, false);
@@ -794,20 +916,20 @@ int QDocumentSearch::next(bool backward, bool all, bool again, bool allowWrapAro
 			int ret = QMessageBox::Yes; //different to qce, see above
 			if ( !hasOption(Silent) )
 				ret=QMessageBox::question(
-								m_editor,
-								tr("Replacing Finished"),
-								tr("%1 (of %2) occurences have been replaced").arg(replaceCount).arg(foundCount)+"\n\n"+
-								tr("Do you want to continue replacing from the beginning?"),
-								QMessageBox::Yes
-								| QMessageBox::No,
-								QMessageBox::Yes
-							);
+				      m_editor,
+				      tr("Replacing Finished"),
+				      tr("%1 (of %2) occurrences have been replaced").arg(replaceCount).arg(foundCount)+"\n\n"+
+				      tr("Do you want to continue replacing from the beginning?"),
+				      QMessageBox::Yes
+				      | QMessageBox::No,
+				      QMessageBox::Yes
+				      );
 			if ( ret == QMessageBox::Yes )
 			{
 				QDocumentCursor newScope(backward?firstMatch.selectionEnd():firstMatch.selectionStart());
 				if ( !backward ) newScope.movePosition(0,QDocumentCursor::Start,QDocumentCursor::KeepAnchor);
 				else newScope.movePosition(0,QDocumentCursor::End,QDocumentCursor::KeepAnchor);
-				if (scope.isValid() && scope.hasSelection()) newScope = scope.intersect(newScope);			
+				if (scope.isValid() && scope.hasSelection()) newScope = scope.intersect(newScope);
 				m_cursor = QDocumentCursor();
 
 				int result = foundCount;
@@ -817,7 +939,7 @@ int QDocumentSearch::next(bool backward, bool all, bool again, bool allowWrapAro
 				return result;
 			}
 		} else if ( !hasOption(Silent) )
-			QMessageBox::information(m_editor,tr("Replacing Finished"),tr("%1 (of %2) occurences have been replaced").arg(replaceCount).arg(foundCount),QMessageBox::Ok);
+			QMessageBox::information(m_editor,tr("Replacing Finished"),tr("%1 (of %2) occurrences have been replaced").arg(replaceCount).arg(foundCount),QMessageBox::Ok);
 	}
 	
 	
@@ -835,7 +957,7 @@ void QDocumentSearch::replaceCursorText(bool backward){
 	m_lastReplacedPosition = m_cursor;
 	
 	//make sure that the cursor if  the correct side of the selection is used
-	//(otherwise the cursor could be moved out of the searched scope by a long 
+	//(otherwise the cursor could be moved out of the searched scope by a long
 	//replacement text)
 	if (m_cursor.hasSelection()) {
 		if (backward) m_cursor=m_cursor.selectionStart();
@@ -851,7 +973,7 @@ void QDocumentSearch::updateReplacementOverlays(){
 		return;
 	}
 	int rid = d->getFormatId("replacement");
-    m_editor->removeMark("replace");
+	m_editor->removeMark("replace");
 	if (!hasOption(HighlightReplacements) || !rid)  {
 		m_newReplacementOverlays.clear();
 		return;
@@ -861,15 +983,15 @@ void QDocumentSearch::updateReplacementOverlays(){
 		QDocumentLine startLine = d->line(boundaries.startLine);
 		QDocumentLine endLine = d->line(boundaries.endLine);
 		m_highlightedReplacements.insert(startLine.handle());
-        if (boundaries.startLine == boundaries.endLine){  //single line replacement
+		if (boundaries.startLine == boundaries.endLine){  //single line replacement
 			startLine.addOverlay(QFormatRange(boundaries.start, boundaries.end - boundaries.start, rid));
-            m_editor->addMark(boundaries.startLine,Qt::red,"replace");
+			m_editor->addMark(boundaries.startLine,Qt::red,"replace");
 		} else {
 			//multi line replacement
 			m_highlightedReplacements.insert(endLine.handle());
 			startLine.addOverlay(QFormatRange(boundaries.start, startLine.length() - boundaries.start, rid));
 			endLine.addOverlay(QFormatRange(0, boundaries.end, rid));
-            m_editor->addMarkRange(boundaries.startLine,boundaries.endLine,Qt::red,"replace");
+			m_editor->addMarkRange(boundaries.startLine,boundaries.endLine,Qt::red,"replace");
 			for (int i=boundaries.startLine+1; i<boundaries.endLine; i++){
 				QDocumentLine curLine = d->line(i);
 				m_highlightedReplacements.insert(curLine.handle());
@@ -884,12 +1006,12 @@ void QDocumentSearch::updateReplacementOverlays(){
 	int firstLine = m_newReplacementOverlays.first().first.startLine;
 	int endLine = qMin(m_newReplacementOverlays.last().first.endLine, d->lineCount()-1);
 	for (int l = firstLine; l<=endLine; l++) {
-		
+
 	}
 
 	QDocumentSelection boundaries = m_cursor.selection();
 			shift = -boundaries.end;
-	
+
 	QDocumentLine startLine = d->line(boundaries.startLine);
 	QDocumentLine endLine = d->line(boundaries.endLine);
 			oldOverlaysBefore = startLine.getOverlays(rid);
@@ -924,7 +1046,7 @@ void QDocumentSearch::updateReplacementOverlays(){
 
 void QDocumentSearch::documentContentChanged(int line, int n){
 	if (!m_editor || !m_editor->document()) return;
-	if (m_scope.isValid() && m_scope.hasSelection() && m_scope.selectionStart()==m_scope.selectionEnd()) 
+	if (m_scope.isValid() && m_scope.hasSelection() && m_scope.selectionStart()==m_scope.selectionEnd())
 		setScope(QDocumentCursor());
 	if(!hasOption(HighlightAll)) {
 		highlightSelection();
